@@ -1,5 +1,6 @@
 from typing import Any, Dict, Set, Union
 
+from strong_typing.core import JsonType
 from strong_typing.docstring import parse_type
 from strong_typing.inspection import (
     is_generic_list,
@@ -44,26 +45,18 @@ from .specification import (
 )
 
 
-class Generator:
-    endpoint: type
-    options: Options
+SchemaOrRef = Union[Schema, SchemaRef]
+
+
+class SchemaBuilder:
     schema_generator: JsonSchemaGenerator
     schemas: Dict[str, Schema]
-    responses: Dict[str, Response]
 
-    def __init__(self, endpoint: type, options: Options) -> None:
-        self.endpoint = endpoint
-        self.options = options
-        self.schema_generator = JsonSchemaGenerator(
-            SchemaOptions(
-                definitions_path="#/components/schemas/",
-                property_description_fun=options.property_description_fun,
-            )
-        )
+    def __init__(self, schema_generator: JsonSchemaGenerator) -> None:
+        self.schema_generator = schema_generator
         self.schemas = {}
-        self.responses = {}
 
-    def _classdef_to_schema(self, typ: type) -> Schema:
+    def classdef_to_schema(self, typ: type) -> Schema:
         """
         Converts a type to a JSON schema.
         For nested types found in the type hierarchy, adds the type to the schema registry in the OpenAPI specification section `components`.
@@ -73,18 +66,22 @@ class Generator:
 
         # append schema to list of known schemas, to be used in OpenAPI's Components Object section
         for ref, schema in type_definitions.items():
-            if ref not in self.schemas:
-                self.schemas[ref] = schema
+            self._add_ref(ref, schema)
 
         return type_schema
 
-    def _classdef_to_ref(self, typ: type) -> Union[Schema, SchemaRef]:
+    def classdef_to_named_schema(self, name: str, typ: type) -> Schema:
+        schema = self.classdef_to_schema(typ)
+        self._add_ref(name, schema)
+        return schema
+
+    def classdef_to_ref(self, typ: type) -> SchemaOrRef:
         """
         Converts a type to a JSON schema, and if possible, returns a schema reference.
         For composite types (such as classes), adds the type to the schema registry in the OpenAPI specification section `components`.
         """
 
-        type_schema = self._classdef_to_schema(typ)
+        type_schema = self.classdef_to_schema(typ)
         if typ is str or typ is int or typ is float:
             # represent simple types as themselves
             return type_schema
@@ -102,32 +99,30 @@ class Generator:
         return type_schema
 
     def _build_ref(self, type_name: str, type_schema: Schema) -> SchemaRef:
-        if type_name not in self.schemas:
-            self.schemas[type_name] = type_schema
+        self._add_ref(type_name, type_schema)
         return SchemaRef(type_name)
 
-    def _build_media_type(
-        self, item_type: type, examples: List[Any] = None
-    ) -> MediaType:
-        return MediaType(
-            schema=self._classdef_to_ref(item_type),
-            examples=self._build_examples(examples),
-        )
+    def _add_ref(self, type_name: str, type_schema: Schema) -> None:
+        if type_name not in self.schemas:
+            self.schemas[type_name] = type_schema
 
-    def _build_examples(
-        self, examples: List[Any] = None
-    ) -> Optional[Dict[str, Union[Example, ExampleRef]]]:
 
-        return (
-            {
-                str(example): Example(value=object_to_json(example))
-                for example in examples
-            }
-            if examples is not None
-            else None
-        )
+class ContentBuilder:
+    schema_builder: SchemaBuilder
+    schema_transformer: Callable[[SchemaOrRef], SchemaOrRef]
+    sample_transformer: Callable[[JsonType], JsonType]
 
-    def _build_content(
+    def __init__(
+        self,
+        schema_builder: SchemaBuilder,
+        schema_transformer: Callable[[SchemaOrRef], SchemaOrRef] = None,
+        sample_transformer: Callable[[JsonType], JsonType] = None,
+    ) -> None:
+        self.schema_builder = schema_builder
+        self.schema_transformer = schema_transformer  # type: ignore
+        self.sample_transformer = sample_transformer  # type: ignore
+
+    def build_content(
         self, payload_type: type, examples: List[Any] = None
     ) -> Dict[str, MediaType]:
         "Creates the content subtree for a request or response."
@@ -139,7 +134,109 @@ class Generator:
             media_type = "application/json"
             item_type = payload_type
 
-        return {media_type: self._build_media_type(item_type, examples)}
+        return {media_type: self.build_media_type(item_type, examples)}
+
+    def build_media_type(
+        self, item_type: type, examples: List[Any] = None
+    ) -> MediaType:
+
+        schema = self.schema_builder.classdef_to_ref(item_type)
+        if self.schema_transformer:
+            schema_transformer: Callable[[SchemaOrRef], SchemaOrRef] = self.schema_transformer  # type: ignore
+            schema = schema_transformer(schema)
+        return MediaType(
+            schema=schema,
+            examples=self._build_examples(examples),
+        )
+
+    def _build_examples(
+        self, examples: List[Any] = None
+    ) -> Optional[Dict[str, Union[Example, ExampleRef]]]:
+
+        if examples is None:
+            return None
+
+        if self.sample_transformer:
+            sample_transformer: Callable[[JsonType], JsonType] = self.sample_transformer  # type: ignore
+        else:
+            sample_transformer = lambda sample: sample
+
+        return {
+            str(example): Example(value=sample_transformer(object_to_json(example)))
+            for example in examples
+        }
+
+
+@dataclass
+class ResponseOptions:
+    """
+    Configuration options for building a response for an operation.
+
+    :param type_descriptions: Maps each response type to a textual description (if available).
+    :param examples: A list of response examples.
+    :param status_catalog: Maps each response type to an HTTP status code.
+    :param default_status_code: HTTP status code assigned to responses that have no mapping.
+    """
+
+    type_descriptions: Dict[type, str]
+    examples: Optional[List[Any]]
+    status_catalog: Dict[type, Union[int, str]]
+    default_status_code: Union[int, str]
+
+
+class ResponseBuilder:
+    content_builder: ContentBuilder
+
+    def __init__(self, content_builder: ContentBuilder) -> None:
+        self.content_builder = content_builder
+
+    def _get_status_responses(self, options: ResponseOptions) -> Dict[str, List[type]]:
+        status_responses: Dict[str, List[type]] = {}
+
+        for response_type in options.type_descriptions.keys():
+            status_code = str(
+                options.status_catalog.get(response_type, options.default_status_code)
+            )
+            if status_code not in status_responses:
+                status_responses[status_code] = []
+            status_responses[status_code].append(response_type)
+
+        return status_responses
+
+    def build_response(
+        self, options: ResponseOptions
+    ) -> Dict[str, Union[Response, ResponseRef]]:
+        """
+        Groups responses that have the same status code.
+        """
+
+        responses: Dict[str, Union[Response, ResponseRef]] = {}
+        status_responses = self._get_status_responses(options)
+        for status_code, response_type_list in status_responses.items():
+            response_type_tuple = tuple(response_type_list)
+            if len(response_type_tuple) > 1:
+                composite_response_type: type = Union[response_type_tuple]  # type: ignore
+            else:
+                (response_type,) = response_type_tuple
+                composite_response_type = response_type
+
+            description = " **OR** ".join(
+                filter(
+                    None,
+                    (
+                        options.type_descriptions[response_type]
+                        for response_type in response_type_tuple
+                    ),
+                )
+            )
+
+            responses[status_code] = self._build_response(
+                response_type=composite_response_type,
+                description=description,
+                examples=options.examples if options.examples else None,
+            )
+
+        return responses
 
     def _build_response(
         self, response_type: type, description: str, examples: List[Any] = None
@@ -149,74 +246,29 @@ class Generator:
         if response_type is not None:
             return Response(
                 description=description,
-                content=self._build_content(response_type, examples),
+                content=self.content_builder.build_content(response_type, examples),
             )
         else:
             return Response(description=description)
 
-    def _build_response_group(
-        self,
-        response_type_descriptions: Dict[type, str],
-        response_examples: Optional[List[Any]],
-        response_status_catalog: Dict[type, Union[int, str]],
-        default_status_code: Union[int, str],
-    ) -> Dict[str, Union[Response, ResponseRef]]:
-        """
-        Groups responses that have the same status code.
 
-        :param response_type_descriptions: Maps each response type to a textual description (if available).
-        :param response_examples: A list of response examples.
-        :param response_status_catalog: Maps each response type to an HTTP status code.
-        :param default_status_code: HTTP status code assigned to responses that have no mapping.
-        """
+class Generator:
+    endpoint: type
+    options: Options
+    schema_builder: SchemaBuilder
+    responses: Dict[str, Response]
 
-        status_responses: Dict[str, List[type]] = {}
-        for response_type in response_type_descriptions.keys():
-            status_code = str(
-                response_status_catalog.get(response_type, default_status_code)
+    def __init__(self, endpoint: type, options: Options) -> None:
+        self.endpoint = endpoint
+        self.options = options
+        schema_generator = JsonSchemaGenerator(
+            SchemaOptions(
+                definitions_path="#/components/schemas/",
+                property_description_fun=options.property_description_fun,
             )
-            if status_code not in status_responses:
-                status_responses[status_code] = []
-            status_responses[status_code].append(response_type)
-
-        responses: Dict[str, Union[Response, ResponseRef]] = {}
-        for status_code, response_type_list in status_responses.items():
-            response_type_tuple = tuple(response_type_list)
-            if len(response_type_tuple) > 1:
-                composite_response_type: type = Union[response_type_tuple]  # type: ignore
-            else:
-                response_type = response_type_tuple[0]
-                composite_response_type = response_type
-
-            description = " **OR** ".join(
-                filter(
-                    None,
-                    (
-                        response_type_descriptions[response_type]
-                        for response_type in response_type_tuple
-                    ),
-                )
-            )
-
-            if response_examples:
-                if all(isinstance(t, type) for t in response_type_tuple):
-                    examples = [
-                        example
-                        for example in response_examples
-                        if isinstance(example, response_type_tuple)
-                    ]
-                else:
-                    examples = response_examples
-            else:
-                examples = []
-
-            responses[status_code] = self._build_response(
-                response_type=composite_response_type,
-                description=description,
-                examples=examples if examples else None,
-            )
-
-        return responses
+        )
+        self.schema_builder = SchemaBuilder(schema_generator)
+        self.responses = {}
 
     def _build_type_tag(self, ref: str, schema: Schema) -> Tag:
         definition = f'<SchemaDefinition schemaRef="#/components/schemas/{ref}" />'
@@ -244,10 +296,9 @@ class Generator:
             tag_list: List[Tag] = []
 
             for extra_type in category_items:
-                ref = extra_type.__name__
-                type_schema = self._classdef_to_schema(extra_type)
-                self.schemas[ref] = type_schema
-                tag_list.append(self._build_type_tag(ref, type_schema))
+                name = python_type_to_name(extra_type)
+                schema = self.schema_builder.classdef_to_named_schema(name, extra_type)
+                tag_list.append(self._build_type_tag(name, schema))
 
             extra_tags[category_name] = tag_list
 
@@ -266,7 +317,7 @@ class Generator:
                 in_=ParameterLocation.Path,
                 description=doc_params.get(param_name),
                 required=True,
-                schema=self._classdef_to_ref(param_type),
+                schema=self.schema_builder.classdef_to_ref(param_type),
             )
             for param_name, param_type in op.path_params
         ]
@@ -286,7 +337,7 @@ class Generator:
                 in_=ParameterLocation.Query,
                 description=doc_params.get(param_name),
                 required=required,
-                schema=self._classdef_to_ref(inner_type),
+                schema=self.schema_builder.classdef_to_ref(inner_type),
             )
             query_parameters.append(query_parameter)
 
@@ -295,10 +346,11 @@ class Generator:
 
         # data passed in payload
         if op.request_param:
+            builder = ContentBuilder(self.schema_builder)
             request_name, request_type = op.request_param
             requestBody = RequestBody(
                 content={
-                    "application/json": self._build_media_type(
+                    "application/json": builder.build_media_type(
                         request_type, op.request_examples
                     )
                 },
@@ -318,40 +370,77 @@ class Generator:
         else:
             # use return type as a single response type
             success_type_descriptions = {
-                op.response_type: doc_string.returns.description
-                if doc_string.returns
-                else "OK"
+                op.response_type: (
+                    doc_string.returns.description if doc_string.returns else "OK"
+                )
             }
 
-        responses: Dict[str, Union[Response, ResponseRef]] = self._build_response_group(
+        response_examples = op.response_examples or []
+        success_examples = [
+            example
+            for example in response_examples
+            if not isinstance(example, Exception)
+        ]
+
+        content_builder = ContentBuilder(self.schema_builder)
+        response_builder = ResponseBuilder(content_builder)
+        response_options = ResponseOptions(
             success_type_descriptions,
-            op.response_examples,
+            success_examples,
             self.options.success_responses,
             "200",
         )
+        responses = response_builder.build_response(response_options)
 
         # failure response types
         if doc_string.raises:
-            exception_types = {
+            exception_types: Dict[type, str] = {
                 item.raise_type: item.description for item in doc_string.raises.values()
             }
+            exception_examples = [
+                example
+                for example in response_examples
+                if isinstance(example, Exception)
+            ]
 
-            responses.update(
-                self._build_response_group(
-                    exception_types,
-                    op.response_examples,
-                    self.options.error_responses,
-                    "500",
-                )
+            if self.options.error_wrapper:
+                schema_transformer = lambda schema: {
+                    "type": "object",
+                    "properties": {
+                        "error": schema,
+                    },
+                    "additionalProperties": False,
+                    "required": [
+                        "error",
+                    ],
+                }
+                sample_transformer = lambda error: {"error": error}
+            else:
+                schema_transformer = None
+                sample_transformer = None
+
+            content_builder = ContentBuilder(
+                self.schema_builder,
+                schema_transformer=schema_transformer,
+                sample_transformer=sample_transformer,
             )
+            response_builder = ResponseBuilder(content_builder)
+            response_options = ResponseOptions(
+                exception_types,
+                exception_examples,
+                self.options.error_responses,
+                "500",
+            )
+            responses.update(response_builder.build_response(response_options))
 
         if op.event_type is not None:
+            builder = ContentBuilder(self.schema_builder)
             callbacks = {
                 f"{op.func_name}_callback": {
                     "{$request.query.callback}": PathItem(
                         post=Operation(
                             requestBody=RequestBody(
-                                content=self._build_content(op.event_type)
+                                content=builder.build_content(op.event_type)
                             ),
                             responses={"200": Response(description="OK")},
                         )
@@ -413,16 +502,15 @@ class Generator:
 
         # types that are produced/consumed by operations
         type_tags = [
-            self._build_type_tag(ref, schema) for ref, schema in self.schemas.items()
+            self._build_type_tag(ref, schema)
+            for ref, schema in self.schema_builder.schemas.items()
         ]
 
         # types that are emitted by events
         event_tags: List[Tag] = []
         events = get_endpoint_events(self.endpoint)
         for ref, event_type in events.items():
-            event_schema = self._classdef_to_schema(event_type)
-            if ref not in self.schemas:
-                self.schemas[ref] = event_schema
+            event_schema = self.schema_builder.classdef_to_named_schema(ref, event_type)
             event_tags.append(self._build_type_tag(ref, event_schema))
 
         # types that are explicitly declared
@@ -490,7 +578,7 @@ class Generator:
             servers=[self.options.server],
             paths=paths,
             components=Components(
-                schemas=self.schemas,
+                schemas=self.schema_builder.schemas,
                 responses=self.responses,
                 securitySchemes=securitySchemes,
             ),
