@@ -21,6 +21,7 @@ class Options:
     :param extra_types: Extra types in addition to those found in operation signatures. Use a dictionary to group related types.
     :param success_responses: Associates operation response types with HTTP status codes.
     :param error_responses: Associates error response types with HTTP status codes.
+    :param error_wrapper: True if errors are encapsulated in an error object wrapper.
     :param property_description_fun: Custom transformation function to apply to class property documentation strings.
     :param captions: User-defined captions for sections such as "Operations" or "Types", and (if applicable) groups of extra types.
     """
@@ -35,6 +36,7 @@ class Options:
     error_responses: Dict[type, Union[int, str]] = dataclasses.field(
         default_factory=dict
     )
+    error_wrapper: bool = False
     property_description_fun: Optional[Callable[[type, str, str], str]] = None
     captions: Optional[Dict[str, str]] = None
 
